SystemOrganization addCategory: #'WebClient-Seaside'!

WAServerAdaptor subclass: #WAWebServerAdaptor
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebClient-Seaside'!

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 5/4/2010 16:32'!
basicStart
	server := WebServer new.
	server addService: '/' action: (MessageSend receiver: self selector: #process:).
	server listenOn: self port.
! !

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 5/4/2010 16:28'!
basicStop
	server ifNotNil:[
		server destroy.
		server := nil.
	].! !

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 5/4/2010 16:39'!
decodeFields: aDictionary using: aOneArgumentBlock into: aRequestFields
	aDictionary keysAndValuesDo: [ :key :value |
		value isString
			ifTrue: [ aRequestFields at: key add: (aOneArgumentBlock value: value) ]
			ifFalse: [
				(value isCollection)
					ifTrue: [ value do: [ :string |
						aRequestFields at: key add: (aOneArgumentBlock value: string) ] ]
					ifFalse: [ aRequestFields at: key add: value ] ] ]! !

!WAWebServerAdaptor methodsFor: 'testing' stamp: 'ar 5/4/2010 16:29'!
isRunning
	^server notNil and:[server isRunning]! !

!WAWebServerAdaptor methodsFor: 'testing' stamp: 'ar 5/4/2010 16:29'!
isStarting
	^false! !

!WAWebServerAdaptor methodsFor: 'testing' stamp: 'ar 5/4/2010 16:29'!
isStopped
	^server isNil or:[server isRunning not]! !

!WAWebServerAdaptor methodsFor: 'testing' stamp: 'ar 5/4/2010 16:22'!
isStopping
	^false! !

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 5/4/2010 16:38'!
isUrlEncoded: aNativeRequest
	"answers whether the body of the given request is url encoded"
	^aNativeRequest contentType = 'application/x-www-form-urlencoded'! !

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 5/4/2010 16:38'!
postFieldsOf: aNativeRequest into: aRequestFields
	"puts all the POST parameters of the given request decoded into a WARequestFields"
	| isUrlEncoded |
	isUrlEncoded := self isUrlEncoded: aNativeRequest.
	^self
		decodeFields: aNativeRequest postFields
		using: [ :each |
			isUrlEncoded 
				ifTrue: [ self codec url decode: each ]
				ifFalse: [ self codec decode: each ] ]
		into: aRequestFields! !

!WAWebServerAdaptor methodsFor: 'processing' stamp: 'ar 5/4/2010 20:44'!
process: webRequest
	"Process the request; send the response back to the client"

	| webResponse |
	webResponse := super process: webRequest.
	webRequest sendResponse: webResponse content: webResponse content.
! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:55'!
requestAddressFor: webRequest
	"Answer the remote socket address as a string"

	^webRequest stream socket remoteAddress reduce:[:a :b| a asString,'.', b].! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:54'!
requestBodyFor: webRequest
	"Answer the undecoded request body as a String."

	^(webRequest contentLength = nil and:[webRequest isGetRequest])
		ifTrue:[nil]
		ifFalse:[webRequest content]! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:37'!
requestCookiesFor: webRequest

	^ WARequestCookie fromString: (webRequest headerAt: 'cookie' ifAbsent: [ nil ])! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:44'!
requestFieldsFor: webRequest
	"XXX: Verify correctness"
	| fields |
	fields := WARequestFields new.
	webRequest isPostRequest
		ifTrue: [ self postFieldsOf: webRequest into: fields ].
	^ fields! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:44'!
requestHeadersFor: webRequest
	"XXX: Seems odd. It appears we must return a single-valued dictionary here
	but what if there are multiple headers?"
	| dict |
	dict := Dictionary new.
	webRequest headersDo:[:key :val| dict at: key put: val].
	^dict! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:45'!
requestMethodFor: webRequest
	"Answers the request method (GET, POST etc)"

	^webRequest method! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:46'!
requestUrlFor: webRequest
	"Answers a WAUrl from the given request"

	^(WAUrl absolute: webRequest rawUrl)
		decodedWith: self codec! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:46'!
requestVersionFor: webRequest
	"Answers the WAHttpVersion from the request protocol"

	^ WAHttpVersion fromString: webRequest protocol! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 20:44'!
responseFrom: aRequestContext
	"Answer a native response from aRequestContext."

	| response |
	response := WebResponse new.
	response protocol: aRequestContext request version asString 
			code: aRequestContext response status.
	aRequestContext response headers keysAndValuesDo:[ :key :value | 
		response headerAt: key put: value greaseString.
	].
	aRequestContext response cookies do:[:cookie|
		response headerAt: 'set-cookie' put: cookie oldNetscapeString.
	].
	response content: aRequestContext response stream contents.
	^response! !

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 5/4/2010 16:27'!
shutDown
! !

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 5/4/2010 16:27'!
startUp
! !
